@@ -132,18 +132,20 @@ extern void addBtUuid(QBluetoothUuid uuid);
 extern QHash<QString, QStringList> productList;
 extern QStringList vendorList;
 
+QString btDeviceAddress(const QBluetoothDeviceInfo *device, bool isBle)
+{
+	QString address = device->address().isNull() ?
+		device->deviceUuid().toString() : device->address().toString();
+	const char *prefix = isBle ? "LE:" : "";
+	return prefix + address;
+}
+
 QString markBLEAddress(const QBluetoothDeviceInfo *device)
 {
-	QBluetoothDeviceInfo::CoreConfigurations flags;
-	QString prefix = "";
+	QBluetoothDeviceInfo::CoreConfigurations flags = device->coreConfigurations();
+	bool isBle = flags == QBluetoothDeviceInfo::LowEnergyCoreConfiguration;
 
-	flags = device->coreConfigurations();
-	if (flags == QBluetoothDeviceInfo::LowEnergyCoreConfiguration)
-		prefix = "LE:";
-	if (device->address().isNull())
-		return prefix + device->deviceUuid().toString();
-	else
-		return prefix + device->address().toString();
+	return btDeviceAddress(device, isBle);
 }
 
 void BTDiscovery::btDeviceDiscovered(const QBluetoothDeviceInfo &device)
@@ -160,6 +162,15 @@ void BTDiscovery::btDeviceDiscovered(const QBluetoothDeviceInfo &device)
 		qDebug() << id.toByteArray();
 	}
 
+#if defined(Q_OS_IOS)
+	// On Desktop this is called when "Save" button is clicked. All
+	// DeviceInfo are stored as data on the ui list items.
+	// On mobile (iOS) the current ConnectionListModel does not support
+	// additional data, so just save all discovered devices.
+
+	saveBtDeviceInfo(btDeviceAddress(&device, false).toUtf8().constData(), device);
+#endif
+
 	btDeviceDiscoveredMain(this_d);
 #endif
 }
@@ -184,7 +195,7 @@ void BTDiscovery::btDeviceDiscoveredMain(const btPairedDevice &device)
 		btVP.vendorIdx = vendorList.indexOf(vendor);
 		btVP.productIdx = productList[vendor].indexOf(newDevice);
 		btDCs << btVP;
-		connectionListModel.addAddress(device.address + " (" + newDevice + ")");
+		connectionListModel.addAddress(newDevice + " " + device.address);
 		return;
 	}
 	connectionListModel.addAddress(device.address);
