@@ -7,11 +7,10 @@ extern "C" {
 #endif
 
 extern double buehlmann_N2_t_halflife[];
-extern struct deco_state *deco_state;
 
 extern int deco_allowed_depth(double tissues_tolerance, double surface_pressure, struct dive *dive, bool smooth);
 
-double get_gf(double ambpressure_bar, const struct dive *dive);
+double get_gf(struct deco_state *ds, double ambpressure_bar, const struct dive *dive);
 
 #ifdef __cplusplus
 }
